<?php
namespace App\Controller;

use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;

class Order {

    private $orderService;
    private $util;

    public function __construct(
        \App\Service\Order $orderService,
        \App\Util\Util $util
    ) {

        $this->orderService = $orderService;
        $this->util = $util;
    }

    public function history(Request $request, Response $response)
    {
        try {
            $params = $request->getParsedBody();

            if (empty($params['dtInicial']) || empty($params['dtFinal'])) {
                throw new \Exception("Data inicial/final não informada.");
            }

            $idStComanda = !empty($params['idStComanda']) ? implode(',', $params['idStComanda']) : 'T';
            $idSincAgenda = $params['idSincAgenda'] ?? 'T';

            $isScheduled = false;

            if($idSincAgenda === 'N'){
                $idStComanda = 'A';
                $isScheduled = true;
            }

            $dtInicial = $params['dtInicial'];
            $dtFinal = $params['dtFinal'];
            $dtIniAgendada = !empty($params['dtIniAgendada']) ? $params['dtIniAgendada'] : 'T';
            $dtFimAgendada= !empty($params['dtFimAgendada']) ? $params['dtFimAgendada'] : 'T';
            $cdFilial = !empty($params['cdFilial']) ? implode(',', $params['cdFilial']) : 'T';
            $nrVendaRest = !empty($params['nrVendaRest']) ? implode(',', $params['nrVendaRest']): 'T';
            $nrComanda = !empty($params['nrComanda']) ? implode(',', $params['nrComanda']): 'T';
            $clienteConsumidor = !empty($params['clienteConsumidor']) ? implode(',', $params['clienteConsumidor']) : 'T';
            $idSincMsde = !empty($params['idSincMsde']) ? implode(',', $params['idSincMsde']) : 'T';
            $idRetBalLoja = !empty($params['idRetBalLoja']) ? implode(',', $params['idRetBalLoja']) : 'T';
            $idOrgCmdVenda = !empty($params['idOrgCmdVenda']) ? implode(',', $params['idOrgCmdVenda']) : 'T';
            $ordersType = !empty($params['ordersType']) ? implode(',', $params['ordersType']) : 'T';
            $scheduledTab = !empty($params['scheduledTab']) ? implode(',', $params['scheduledTab']) : 'T'; 
            $paramsHistory = [
                'dtInicial'         => $dtInicial,
                'dtFinal'           => $dtFinal,
                'dtIniAgendada'     => $dtIniAgendada,
                'dtFimAgendada'     => $dtFimAgendada,
                'cdFilial'          => $cdFilial,
                'nrVendaRest'       => $nrVendaRest,
                'nrComanda'         => $nrComanda,
                'idStComanda'       => $idStComanda,
                'clienteConsumidor' => $clienteConsumidor,
                'idSincAgenda'      => $idSincAgenda,
                'idSincMsde'        => $idSincMsde,
                'idRetBalLoja'      => $idRetBalLoja,
                'idOrgCmdVenda'     => $idOrgCmdVenda,
                'ordersType'        => $ordersType,
                'scheduledTab'      => $scheduledTab,
            ];

            $dataorder = $this->orderService->history($paramsHistory, $isScheduled);

            if(empty($dataorder)) {
                throw new \Exception("Nenhum pedido encontrado.");
            }

            $response->getBody()->write(json_encode([
                'error' => false,
                'message' => '',
                'data' => $dataorder,
                'pagination' => [
                    'limit' => $params['limit'] ?? 50,
                    'total' => count($dataorder),
                    'page' => $params['page'] ?? 1
                ]
            ]));
    
            return $response->withHeader('Content-Type', 'application/json');
        } catch (\Exception $e) {
            $response->getBody()->write(json_encode([
                'error'   => true,
                'message' => $e->getMessage(),
                'data'    => []
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        }
    }

    public function errorLogs(Request $request, Response $response)
    {
        try {
            $params = $request->getParsedBody();
            $uriQueryParams = $request->getUri()->getQuery();

            $cdfilial = $params['CDFILIAL'] ?? null;
            $cdintegracao = $params['CDINTEGRACAO'] ?? null;
            $search = $params['search'] ?? null;
            
            $this->orderService->checkOldLogs();
            
            $datalogs = $this->orderService->errorLogs($cdfilial, $cdintegracao, $search, $uriQueryParams);

            $response->getBody()->write(json_encode([
                'error' => false,
                'message' => '',
                'data' => $datalogs,
                'pagination' => [
                    'limit' => $params['limit'] ?? 10,
                    'total' => count($datalogs),
                    'page' => $params['page'] ?? 1
                ]
            ]));
    
            return $response->withHeader('Content-Type', 'application/json');
        } catch (\Exception $e) {
            $response->getBody()->write(json_encode([
                'error'   => true,
                'message' => $e->getMessage(),
                'data'    => []
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        }
    }

    public function cancel(Request $request, Response $response)
    {
        try {
            $params = $request->getParsedBody();
            $cdFilial   = !empty($params['CDFILIAL']) ? $params['CDFILIAL'] : null;
            $cdOperador = !empty($params['CDOPERADOR']) ? $params['CDOPERADOR'] : null;
            $nrComanda = !empty($params['NRCOMANDA']) ? $params['NRCOMANDA'] : null;
            $nrVendaRest = !empty($params['NRVENDAREST']) ? $params['NRVENDAREST'] : null;
            $dsCancMotiv = !empty($params['DSCANCMOTIV']) ? $params['DSCANCMOTIV'] : null;

            if(empty($cdFilial) || empty($cdOperador) || empty($nrComanda) || empty($nrVendaRest) || empty($dsCancMotiv)) {
                throw new \Exception("Algum parâmetro não foi informado.");
            }
            
            $this->orderService->cancel($cdFilial, $cdOperador, $nrComanda, $nrVendaRest, $dsCancMotiv);

            $response->getBody()->write(json_encode([
                'error'   => false,
                'message' => 'Pedido cancelado.',
                'data'    => []
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        } catch (\Exception $e) {
            $response->getBody()->write(json_encode([
                'error'   => true,
                'message' => $e->getMessage(),
                'data'    => []
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        }
    }

    public function getBranchList(Request $request, Response $response) {
        try {
            $params = $request->getParsedBody();
            $search = isset($params['search']) ? strval(strtoupper($params['search'])) : '';

            $branchList = $this->orderService->getBranchList();

            if(empty($branchList)) {
                throw new \Exception("Nenhuma filial encontrada.");
            }

            if (!empty($search) && $search !== '') {
                $branchList = array_values(array_filter($branchList, function($value) use ($search) {
                    return (mb_strpos($value['LABELFILIAL'], $search) !== false) || '';
                }));
            }

            $response->getBody()->write(json_encode([
                'error'   => false,
                'message' => '',
                'data'    => $branchList,
                'pagination' => [
                    'limit' => $params['limit'] ?? 10,
                    'total' => count($branchList),
                    'page' => $params['page'] ?? 1
                ]
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        } catch (\Exception $e) {
            $response->getBody()->write(json_encode([
                'error'   => true,
                'message' => $e->getMessage(),
                'data'    => []
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        }
    }

    public function getOrderNumber(Request $request, Response $response) {
        try {
            $params = $request->getQueryParams();
            $search = isset($params['search']) ? strval($params['search']) : '';
            $orderNumber = [];

            if (!empty($search) && strlen($search) > 4) {
                $orderNumber = $this->orderService->getOrderNumber($search);
            }

            $response->getBody()->write(json_encode([
                'error'   => false,
                'message' => '',
                'data'    => $orderNumber,
                'pagination' => [
                    'limit' => $params['limit'] ?? 10,
                    'total' => count($orderNumber),
                    'page' => $params['page'] ?? 1
                    ]
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        } catch (\Exception $e) {
            $response->getBody()->write(json_encode([
                'error'   => true,
                'message' => $e->getMessage(),
                'data'    => []
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        }
    }

    public function deleteLogs(Request $request, Response $response)
    {
        try {
            $params = $request->getParsedBody();

            $CDINTEGRACAOERR = !empty($params['CDINTEGRACAOERR']) ? $params['CDINTEGRACAOERR'] : null;

            if (empty($CDINTEGRACAOERR)) {
                throw new \Exception("Código do log não informado.");
            }

            $this->orderService->deleteLogs($CDINTEGRACAOERR);

            $response->getBody()->write(json_encode([
                'error' => false,
                'message' => 'Log excluído com sucesso',
                'data' => []
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        } catch (\Exception $e) {
            $response->getBody()->write(json_encode([
                'error'   => true,
                'message' => $e->getMessage(),
                'data'    => []
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        }
    }

    public function getConsumers(Request $request, Response $response) 
    {
        try {
            $params = $request->getQueryParams();
            $consumers = $this->orderService->getConsumers($params);

            if (isset($consumers)) {
            $response->getBody()->write(json_encode([
                'error'   => false,
                'message' => '',
                'data'    => $consumers,
                'pagination' => [
                    'limit' => $params['limit'] ?? 10,
                    'total' => count($consumers),
                    'page' => $params['page'] ?? 1
                ],
            ]));
        }
            return $response->withHeader('Content-Type', 'application/json');
        } catch (\Exception $e) {
            $response->getBody()->write(json_encode([
                'error'   => true,
                'message' => $e->getMessage(),
                'data'    => []
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        }
    }

    public function selectTableInteg(Request $request, Response $response)
    {
        try {
            $params = $request->getParsedBody();

            $dataTableInteg = $this->orderService->selectTableInteg();
    
            $response->getBody()->write(json_encode([
                'error' => false,
                'message' => '',
                'data' => $dataTableInteg,
                'pagination' => [
                    'limit' => $params['limit'] ?? 5,
                    'total' => count($dataTableInteg),
                    'page' => $params['page'] ?? 1
                ]
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        } catch (\Exception $e) {
            $response->getBody()->write(json_encode([
                'error'   => true,
                'message' => $e->getMessage(),
                'data'    => []
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        }
    }

    public function selectTableFilial(Request $request, Response $response)
    {
        try {
            $params = $request->getParsedBody();

            $dataTableFilial = $this->orderService->selectTableFilial();

            $response->getBody()->write(json_encode([
                'error' => false,
                'message' => '',
                'data' => $dataTableFilial,
                'pagination' => [
                    'limit' => $params['limit'] ?? 5,
                    'total' => count($dataTableFilial),
                    'page' => $params['page'] ?? 1
                ]
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        } catch (\Exception $e) {
            $response->getBody()->write(json_encode([
                'error'   => true,
                'message' => $e->getMessage(),
                'data'    => []
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        }
    }

    public function getLogDetails(Request $request, Response $response)
    {
        try {
            $params = $request->getParsedBody();

            $cdIntegracaoErr = !empty($params['CDINTEGRACAOERR']) ? $params['CDINTEGRACAOERR'] : null;

            if (empty($cdIntegracaoErr)) {
                throw new \Exception("Código do log não informado.");
            }

            $dataLogDetails = $this->orderService->getLogDetails($cdIntegracaoErr);

            $response->getBody()->write(json_encode([
                'error' => false,
                'message' => '',
                'data' => $dataLogDetails
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        } catch (\Exception $e) {
            $response->getBody()->write(json_encode([
                'error'   => true,
                'message' => $e->getMessage(),
                'data'    => []
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        }
    }

    public function getOrderDetails(Request $request, Response $response) {
        try {
            $params = $request->getParsedBody();

            $orderDetails = $this->orderService->getOrderDetails($params);

            $response->getBody()->write(json_encode([
                'error' => false,
                'message' => '',
                'data' => $orderDetails
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        } catch (\Exception $e) {
            $response->getBody()->write(json_encode([
                'error'   => true,
                'message' => $e->getMessage(),
                'data'    => []
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        }
    }

    public function getOrderAddress(Request $request, Response $response) {
        try {
            $params = $request->getParsedBody();

            $orderAddress = $this->orderService->getOrderAddress($params);

            $response->getBody()->write(json_encode([
                'error' => false,
                'message' => '',
                'data' => $orderAddress
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        } catch (\Exception $e) {
            $response->getBody()->write(json_encode([
                'error'   => true,
                'message' => $e->getMessage(),
                'data'    => []
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        }
    }

    public function getPaymentMethods(Request $request, Response $response) {
        try {
            $params = $request->getParsedBody();

            $paymentMethods = $this->orderService->getPaymentMethods($params);
            
            $response->getBody()->write(json_encode([
                'error' => false,
                'message' => '',
                'data' => $paymentMethods
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        } catch (\Exception $e) {
            $response->getBody()->write(json_encode([
                'error'   => true,
                'message' => $e->getMessage(),
                'data'    => []
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        }
    }

    public function dashboard(Request $request, Response $response) {
        try {
            $params = $request->getParsedBody();

            $dtInicial = !empty($params['dtInicial']) ? $params['dtInicial'] : null;
            $dtFinal = !empty($params['dtFinal']) ? $params['dtFinal'] : null;

            if (empty($params['dtInicial']) || empty($params['dtFinal'])) {
                throw new \Exception("Data inicial/final não informada.");
            }
            
            $dashboard = $this->orderService->dashboard($dtInicial, $dtFinal);

            $response->getBody()->write(json_encode([
                'error' => false,
                'message' => "Dados do Dashboard adquiridos com sucesso.",
                'data' => $dashboard
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        } catch (\Exception $e) {
            $response->getBody()->write(json_encode([
                'error'   => true,
                'message' => $e->getMessage(),
                'data'    => []
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        }
    }

    public function getOrderItemsRepo(Request $request, Response $response) {
        try {
			$params   = $request->getParsedBody();
            $products = $this->orderService->getOrderItemsRepo($params);

            $response->getBody()->write(json_encode([
                'error' => false,
                'message' => '',
                'data' => $products
            ]));

            return $response->withHeader('Content-Type', 'application/json');
		} catch (\Exception $e) {
            $response->getBody()->write(json_encode([
                'error'   => true,
                'message' => $e->getMessage(),
                'data'    => []
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        }
    }

    public function getSubItensRepo(Request $request, Response $response) {
        try {
            $params = $request->getParsedBody();
            $subItens = $this->orderService->getSubItens($params);

            $response->getBody()->write(json_encode([
                'error' => false,
                'message' => '',
                'data' => $subItens
            ]));

            return $response->withHeader('Content-Type', 'application/json');
		} catch (\Exception $e) {
            $response->getBody()->write(json_encode([
                'error'   => true,
                'message' => $e->getMessage(),
                'data'    => []
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        }
    }

    public function getItemObservation(Request $request, Response $response) {
        try {
            $params = $request->getParsedBody();
            $itemObservation = $this->orderService->getItemObservation($params);

            $response->getBody()->write(json_encode([
                'error' => false,
                'message' => '',
                'data' => $itemObservation
            ]));

            return $response->withHeader('Content-Type', 'application/json');
		} catch (\Exception $e) {
            $response->getBody()->write(json_encode([
                'error'   => true,
                'message' => $e->getMessage(),
                'data'    => []
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        }
    }

    public function forwardOrder(Request $request, Response $response) {
        try {
            $params = $request->getParsedBody();

            $cdFilial   = !empty($params['CDFILIAL']) ? $params['CDFILIAL'] : null;
            $nrComanda = !empty($params['NRCOMANDA']) ? $params['NRCOMANDA'] : null;
            $nrVendaRest = !empty($params['NRVENDAREST']) ? $params['NRVENDAREST'] : null;

            if(empty($cdFilial) || empty($nrComanda) || empty($nrVendaRest)) {
                throw new \Exception("Algum parâmetro não foi informado.");
            }
  
            $forwardOrder = $this->orderService->forwardOrder($cdFilial, $nrComanda, $nrVendaRest);
  
            $response->getBody()->write(json_encode([
                'error' => false,
                'message' => 'Pedido adiantado com sucesso.',
                'data' => $forwardOrder
            ]));

            return $response->withHeader('Content-Type', 'application/json');
          } catch (\Exception $e) {
            $response->getBody()->write(json_encode([
                'error'   => true,
                'message' => $e->getMessage(),
                'data'    => []
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        }
      }
  
}