<?php
namespace App\Service;

class Order {

    private $util;
    private $orderFactory;

    public function __construct(\App\Util\Util $util, 
                                \App\Factory\Order $orderFactory) {

        $this->util = $util;
        $this->orderFactory = $orderFactory;
    }

    public function history($paramsHistory, $isScheduled) {

        $history = $this->orderFactory->history($paramsHistory, $isScheduled);
     
        return $history;
    }

    public function errorLogs($cdfilial, $cdintegracao, $search, $uriQueryParams) {

        $dates = $this->getDateFromQueryString($uriQueryParams);
        $orders = $this->getOrderFromQueryString($uriQueryParams);

        $errorLogs = $this->orderFactory->errorLogs($cdfilial, $cdintegracao, $search, $dates, $orders);

        return $errorLogs;
    }

    private function getOrderFromQueryString($uriQueryParams) {
        preg_match_all('/\border=([^&]+)/', $uriQueryParams, $matches);

        $result = array_map(function ($match) {
            return strtoupper(str_replace('.', ' ', $match));
        }, $matches[1]);

        return $result;
    }

    private function getDateFromQueryString($uriQueryParams)
    {
        preg_match_all('/DTINCLUSAO=([^&]+)/', $uriQueryParams, $matches);
        return array_pad(array_map(fn($date) => str_replace('%2F', '/', $date), $matches[1] ?? []), 2, null);
    }

    public function checkOldLogs() {
        try {
            $this->orderFactory->deleteOldLogs($this->util->getEnv("PROJECT_DELETE_LOG_MONTH"));
            $this->orderFactory->updateOldLogs($this->util->getEnv("PROJECT_UPDATE_LOG_MONTH"));
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function cancel($cdFilial, $cdOperador, $nrComanda, $nrVendaRest, $dsCancMotiv) {
        $orderStatus = $this->orderFactory->getOrderStatus($cdFilial, $nrVendaRest, $nrComanda);

        if(empty($orderStatus[0]['IDSTCOMANDA'])) {
            throw new \Exception("Pedido não encontrado.");
        }
        if ($orderStatus[0]['IDSTCOMANDA'] == "4") {
            throw new \Exception("Pedido já cancelado.");
        }
        if ($orderStatus[0]['IDSTCOMANDA'] == 'P' || $orderStatus[0]['IDSTCOMANDA'] == '5' || $orderStatus[0]['IDSTCOMANDA'] == '6') {
            throw new \Exception("Este pedido já foi impresso. Para cancelá-lo, utilize o Gestão de Vendas.");
        }

        $dsCancMotiv = $this->util->removeAcentos($dsCancMotiv);
        $dsCancMotiv = preg_replace('/[^A-Za-z0-9. -]/', '', $dsCancMotiv);
        $dsCancMotiv = strtoupper($dsCancMotiv);
                
        $this->orderFactory->cancelOrder($cdFilial, $nrVendaRest, $nrComanda, $cdOperador, $dsCancMotiv);
        $this->orderFactory->forceOrderSync($cdFilial, $nrVendaRest);
    }

    public function getBranchList() {

        $branchList = $this->orderFactory->getBranchList();

        return $branchList;
    }

    public function getOrderNumber($search) {

        $orderNumber = $this->orderFactory->getOrderNumber($search);

        return $orderNumber;
    }

    public function deleteLogs($CDINTEGRACAOERR) {
        $this->orderFactory->deleteLogs($CDINTEGRACAOERR);
    }

    public function getConsumers($params) {
        $searchTerm = isset($params['search']) ? $params['search'] : '';
        if (strlen($searchTerm) > 2) {
            return $this->orderFactory->getConsumers($searchTerm);
        }
    }

    public function selectTableInteg() {

        $selectTableInteg = $this->orderFactory->selectTableInteg();

        return $selectTableInteg;
    }

    public function selectTableFilial() {
        $selectTableFilial = $this->orderFactory->selectTableFilial();

        return $selectTableFilial;
    }

    public function getLogDetails($cdIntegracaoErr) {

        $getLogDetails = $this->orderFactory->getLogDetails($cdIntegracaoErr);

        if ($getLogDetails) { 
            $dsArqPed = $getLogDetails[0]['DSARQPED'];
            
            if (empty($dsArqPed)) {
                return '';
            }

            $username = $this->util->getEnv("USERNAME_LOG");
            $password = $this->util->getEnv("PASSWORD_LOG");
            $fixedPartUrl = $this->util->getEnv("FIXED_LOG_URL");

            $completeUrl = $fixedPartUrl . $dsArqPed;

            $authHeader = "Authorization: Basic " . base64_encode("$username:$password");
            $options = [
                'http' => [
                    'header' => $authHeader,
                    'ignore_errors' => true
                ]
            ];
            $context = stream_context_create($options);

            $fileContent = file_get_contents($completeUrl, false, $context);

            $json_data = json_decode($fileContent);

            if ($json_data !== null) {
                return $fileContent;
            } else {
                return '';
            }
        } else {
            throw new \Exception("Nenhum registro encontrado.");
        }
    }

    public function getOrderDetails($params) {
        $valueElementOrderDetails = $this->orderFactory->getOrderDetails($params);

        return $valueElementOrderDetails;
    }

    public function getOrderAddress($params) {
        $orderAddress = $this->orderFactory->getOrderAddress($params);

        return $orderAddress;
    }

    public function getPaymentMethods($params) {
        $paymentMethods = $this->orderFactory->getPaymentMethods($params);
        foreach($paymentMethods as &$orderPayment) {
            if ($orderPayment['IDTIPOMOVIVEDLV'] == 'S') {
                $orderPayment['NMTIPORECE'] = 'TROCO';
            }
        }
        return $paymentMethods;
    }

    public function getOrderItemsRepo($params) {
        $products = array();
        $combos = $this->orderFactory->getOrderCombos($params);
        $items  = $this->orderFactory->getOrderItemsRepo($params);

        foreach ($combos as &$combo) {
            $combo['ISCOMBO'] = true;
            array_push($products, $combo);
        }

        foreach ($items as &$item) {
            $item['ISCOMBO'] = false;
            $item['QTPRODCOMVEN'] = floatval($item['QTPRODCOMVEN']);
            array_push($products, $item);
        }

        foreach ($products as &$product) {
            $product['TOTAL']  = (($product['VRPRECCOMVEN'] * $product['QTPRODCOMVEN']) + $product['VRACRCOMVEN'] - $product['VRDESCCOMVEN']);
        }

        return $products;
    }

    public function getSubItens($params) {
        $subItens = $this->orderFactory->getSubItens($params);
        foreach ($subItens as &$item) {
            $item['TOTAL'] = (($item['VRPRECCOMVEN'] * $item['QTPRODCOMVEN']) - $item['VRDESCCOMVEN'] + $item['VRACRCOMVEN']);
        }
        return $subItens;
    }

    public function getItemObservation($params) {
        $explodedObs       = array();
        $observations      = $this->orderFactory->getItemObservation($params);
        $otherObservation  = $this->orderFactory->getOtherItemObservation($params);

        if ($otherObservation) {
            $explodedObs = explode(";", $otherObservation[0]['DSOCORR']);
            foreach($explodedObs as $obsString) {
                $obsString = trim($obsString);
                $isRegularObservation = false;

                if (!empty($obsString)) {
                    foreach($observations as $observation) {
                        if ($observation['DSOCORR'] == $obsString) {
                            $isRegularObservation = true;
                        }
                    }
                    if (!$isRegularObservation) {
                        $otherObservation['DSOCORR'] = $obsString;
                        array_push($observations,$otherObservation);
                    }
                }
            }
        }
        return $observations;
    }

    public function dashboard($dtInicAberMesa, $dtFimAberMesa) {
        try {
            $dashboard = $this->orderFactory->dashboard($dtInicAberMesa, $dtFimAberMesa);

            return $dashboard;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function forwardOrder($cdFilial, $nrComanda, $nrVendaRest) {            
        $orderStatus = $this->orderFactory->getOrderStatus($cdFilial, $nrVendaRest, $nrComanda);

        if(empty($orderStatus[0]['IDSTCOMANDA'])) {
            throw new \Exception("Pedido não encontrado.");
        }
        if ($orderStatus[0]['IDSTCOMANDA'] == "4") {
            throw new \Exception("Pedido já cancelado.");
        }
        if ($orderStatus[0]['IDSTCOMANDA'] == 'P' || $orderStatus[0]['IDSTCOMANDA'] == '5' || $orderStatus[0]['IDSTCOMANDA'] == '6') {
            throw new \Exception("Este pedido já foi impresso.");
        }

        $this->orderFactory->forwardOrderComandaven($cdFilial, $nrComanda, $nrVendaRest);
        $this->orderFactory->forwardOrderVendarest($cdFilial, $nrVendaRest);
        $this->orderFactory->forceOrderSync($cdFilial, $nrVendaRest);
	}

}