import { alertService } from '@/helpers/AlertService';
import Format from '@/helpers/Format';
import Graph from '@/helpers/Graph';
import DateTime from '@/helpers/DateTime';
import { LoadingService } from '@zeedhi/common';
import { Http } from '@zeedhi/core';
import ms from 'ms';

export class DashboardController {
  public todayMonth = new Date().getMonth();

  public retroMonths = 6;

  public quantCancel: string = '0';

  public quantCancelDiff: string = '0';

  public quantSales: string = '0';

  public quantSalesDiff: string = '0';

  public quantSalesToday: string = '0';

  public valueSales: string = 'R$ 0,00';

  public valueSalesDiff: string = 'R$ 0,00';

  public valueSalesToday: string = 'R$ 0,00';

  public quantCancelToday: string = '0';

  public valueCancelToday: string = 'R$ 0,00';

  public valueCancel: string = 'R$ 0,00';

  public valueCancelDiff: string = 'R$ 0,00';

  public async createDashboard() {
    try {
      LoadingService.show();
      const today = new Date();
      const params = {
        dtInicial: this.getMonthsAgo(6),
        dtFinal: `${today.getDate()}/${today.getMonth() + 1}/${today.getFullYear()} 23:59:59`,
      };

      const dashboard = await Http.post('/order/dashboard', params);
      dashboard.data.data.forEach((element: { IDORGCMDVENDA: any; }) => {
        element.IDORGCMDVENDA = Format.apply(element.IDORGCMDVENDA, 'dictionary');
      });

      this.createGraphSalesHour(dashboard);
      this.createGraphSalesToday(dashboard);
      this.createGraphSalesSevenDays(dashboard);
      this.createGraphSalesLastMonth(dashboard);
      this.createGraphSalesMonthModality(dashboard);
      this.createGraphSalesMonthOrigin(dashboard);
      this.createGraphSalesMonthOriginCancel(dashboard);
    } catch (error) {
      if (error instanceof Error) {
        alertService('error', error.message, 'red');
      }
    } finally {
      LoadingService.hide();
    }
  }

  public createGraphSalesToday(dashboard: any) {
    try {
      const today = new Date();
      const salesToday: any[] = [];
      let quantCancelToday = 0;
      let valueCancelToday = 0;
      let valueSalesToday = 0;
      dashboard.data.data.forEach((element: any) => {
        const [day, month, year] = this.getDayMonthYear(element.DTHRABERMESA);
        if (parseInt(day, 10) === today.getDate() && parseInt(month, 10) === today.getMonth() + 1
          && parseInt(year, 10) === today.getFullYear()) {
          if (element.IDSTCOMANDA !== '4') {
            salesToday.push(element);
            valueSalesToday += parseFloat(element.VRPEDIDO);
          } else {
            quantCancelToday += 1;
            valueCancelToday += parseFloat(element.VRPEDIDO);
          }
        }
      });

      const seriesInnerData: any[] = Object.entries(salesToday.reduce((acc: { [x: string]: number; },
        cv: { TOGO: string | number; }) => {
        acc[cv.TOGO] = acc[cv.TOGO] || 0;
        acc[cv.TOGO] += 1;
        return acc;
      }, {}));
      seriesInnerData.forEach((element: any) => {
        element[0] = `${element[0]} (${element[1]})`;
      });

      const seriesOuterData: any[] = Object.entries(salesToday.reduce((acc: { [x: string]: number; },
        cv: { IDORGCMDVENDA: string | number; }) => {
        acc[cv.IDORGCMDVENDA] = acc[cv.IDORGCMDVENDA] || 0;
        acc[cv.IDORGCMDVENDA] += 1;
        return acc;
      }, {}));
      seriesOuterData.forEach((element: any) => {
        element[0] = `${element[0]} (${element[1]})`;
      });

      const datasource = [seriesInnerData, seriesOuterData];
      const title = 'Pedidos Hoje';
      const name = 'sales today';
      const id = 'dashboardCard13';
      const color = ['#5470c6', '#91cc75', '#fac858', '#ee6666', '#73c0de', '#3ba272', '#fc8452', '#9a60b4', '#ea7ccc'];

      const params = {
        datasource,
        title,
        name,
        id,
        color,
      };

      this.createCardValueSalesToday(valueSalesToday);
      this.createCardValueCancelToday(valueCancelToday);
      this.createCardQuantSalesToday(salesToday.length);
      this.createCardQuantCancelToday(quantCancelToday);
      Graph.createGraphPieNest(params);
    } catch (error) {
      if (error instanceof Error) {
        alertService('error', error.message, 'red');
      }
    }
  }

  public createGraphSalesSevenDays(dashboard: any) {
    try {
      const today = new Date();
      const zero = 0;
      today.setHours(zero);
      today.setMinutes(zero);
      today.setSeconds(zero);
      today.setMilliseconds(zero);
      const oneDay = ms('1d');
      const weekDays = 8;
      type SummaryValuesInWeek = [number][];
      type OrderInfo = {
        IDSTCOMANDA: string,
        VRPEDIDO: string,
        DTHRABERMESA: string;
      };
      const salesSevenDays: any[] = Object.entries(dashboard.data.data.reduce((acc: SummaryValuesInWeek,
        cv: OrderInfo) => {
        for (let i = 0; i < weekDays; i += 1) {
          acc[0] = acc[0] || {};
          acc[1] = acc[1] || {};
          acc[0][i] = acc[0][i] || 0;
          acc[1][i] = acc[1][i] || 0;
        }
        const [day, month, year] = this.getDayMonthYear(cv.DTHRABERMESA);
        const saleDay = new Date(`${parseInt(month, 10)}/${day}/${year}`);
        const saleDayDiff = (today.getTime() - saleDay.getTime()) / oneDay;
        if (cv.IDSTCOMANDA !== '4') {
          acc[0][saleDayDiff] = acc[0][saleDayDiff] || 0;
          acc[0][saleDayDiff] += 1;
          acc[1][saleDayDiff] = acc[1][saleDayDiff] || 0;
          acc[1][saleDayDiff] += parseFloat(cv.VRPEDIDO);
        }
        return acc;
      }, {}));

      const newItem = ['Dia', 'Vendas', 'Valores'];
      const datasource = [];
      for (let countDays = 1; countDays < weekDays; countDays += 1) {
        const newDate = new Date(today.getTime() - countDays * oneDay);
        let quantSalesDay = 0;
        let valueSalesDay = 0;
        if (salesSevenDays.length > 0) {
          quantSalesDay = salesSevenDays[0][1][countDays];
          valueSalesDay = salesSevenDays[1][1][countDays].toFixed(2);
        }
        datasource.push([`${newDate.getDate()}/${newDate.getMonth() + 1}`, `${quantSalesDay}`,
          `${valueSalesDay}`]);
      }
      datasource.reverse();
      datasource.unshift(newItem);
      const title = 'Pedidos/Valores dos Últimos 7 Dias';
      const name = 'Seven Days';
      const id = 'dashboardCard14';
      const color = ['#e9561c', '#91cc75'];
      const tooltip = '';
      const yAxis = [
        {
          name: 'Pedidos',
          type: 'value',
        },
        {
          name: 'Valor',
          type: 'value',
          axisLabel: {
            formatter: 'R$ {value}',
          },
        },
      ];

      const params = {
        datasource,
        title,
        name,
        id,
        color,
        tooltip,
        yAxis,
      };
      Graph.createGraphBar(params);
    } catch (error) {
      if (error instanceof Error) {
        alertService('error', error.message, 'red');
      }
    }
  }

  public createGraphSalesLastMonth(dashboard: any) {
    try {
      const today = new Date();
      const todayYear = today.getFullYear();
      const salesLastMonth: any = [];
      dashboard.data.data.forEach((element: any) => {
        const [day, month, year] = this.getDayMonthYear(element.DTHRABERMESA);
        if (day && parseInt(month, 10) === this.todayMonth && parseInt(year, 10) === todayYear
          && element.IDSTCOMANDA !== '4') {
          salesLastMonth.push(element);
        }
      });
      let value = 0;
      salesLastMonth.forEach((element: any) => {
        value += parseFloat(element.VRPEDIDO);
      });

      const PreviousMonthParams = {
        months: 1,
      };
      const datasource = [['product', 'Vendas', 'Valores'], [DateTime.getPreviousMonths(PreviousMonthParams),
        salesLastMonth.length, value.toFixed(2)]];
      const title = 'Pedidos/Valores do Último Mês';
      const name = 'Last Month';
      const id = 'dashboardCard15';
      const yAxis = [
        {
          name: 'Pedidos',
          type: 'value',
        },
        {
          name: 'Valor',
          type: 'value',
          axisLabel: {
            formatter: 'R$ {value}',
          },
        },
      ];
      const color = ['#e9561c', '#91cc75'];
      const tooltip = '';
      const params = {
        datasource,
        title,
        name,
        id,
        color,
        tooltip,
        yAxis,
      };
      Graph.createGraphBar(params);
    } catch (error) {
      if (error instanceof Error) {
        alertService('error', error.message, 'red');
      }
    }
  }

  public createGraphSalesMonthModality(dashboard: any) {
    try {
      const PreviousMonthParams = {
        months: this.retroMonths,
        includeCurrentMonth: true,
      };
      const data = DateTime.getPreviousMonths(PreviousMonthParams);
      const legendData: any = [];
      const today = new Date();
      const todayYear = today.getFullYear();
      const datasource: any = [];
      let totalValue: number = 0;
      let quantSales: number = 0;
      const totalValuePerMonth: number[] = this.createArray(12);
      const nextMonth = new Date();
      nextMonth.setDate(1);
      nextMonth.setMonth(nextMonth.getMonth() - this.retroMonths);

      const salesModalityPerMonth = dashboard.data.data.reduce((acc: any, cv: any) => {
        const [day, month, year] = this.getDayMonthYear(cv.DTHRABERMESA);
        const intMonth = parseInt(month, 10) - 1;
        acc[intMonth] = acc[intMonth] || {};
        if (day && month && parseInt(year, 10) === todayYear && cv.IDSTCOMANDA !== '4') {
          if (!legendData.includes(cv.TOGO)) {
            const newSeries = {
              name: `${cv.TOGO}`,
              type: 'line',
              data: this.createArray(12),
              label: {
                show: true,
                position: 'top',
                color: 'black',
                fontSize: 12,
                formatter(d: { data: string; }) {
                  return `${d.data}`;
                },
              },
            };
            datasource.push(newSeries);
            legendData.push(cv.TOGO);
          }
          acc[intMonth][cv.TOGO] = acc[intMonth][cv.TOGO] + 1 || 1;
          totalValue += parseFloat(cv.VRPEDIDO);
          totalValuePerMonth[parseInt(month, 10) - 1] += parseFloat(cv.VRPEDIDO);
        }
        if (cv.IDSTCOMANDA !== '4') {
          quantSales += 1;
        }
        return acc;
      }, {});
      const totalMonthCancel: number[] = this.createArray(12);
      let k = 0;
      for (nextMonth; nextMonth.getMonth() !== this.todayMonth + 1; nextMonth.setMonth(nextMonth.getMonth() + 1), k += 1) {
        let j = 0;
        for (j; j < legendData.length; j += 1) {
          if (salesModalityPerMonth[nextMonth.getMonth()]) {
            totalMonthCancel[nextMonth.getMonth()] += salesModalityPerMonth[nextMonth.getMonth()][legendData[j]] || 0;
            datasource[j].data[k] = salesModalityPerMonth[nextMonth.getMonth()][legendData[j]] || 0;
          } else {
            datasource[j].data[k] = 0;
          }
        }
        data[k] = data[k].concat(`(${totalMonthCancel[nextMonth.getMonth()]})`);
      }

      const prevMonth = new Date();
      prevMonth.setMonth(prevMonth.getMonth() - 1);

      const title = 'Pedidos Mensais por Modalidade';
      const name = 'sales modality per month';
      const id = 'dashboardCard18';
      const tooltip = '';

      const params = {
        datasource,
        title,
        name,
        id,
        tooltip,
        legendData,
        data,
      };

      this.createCardQuantSales(quantSales);
      this.createCardQuantSalesDiff(totalMonthCancel[prevMonth.getMonth()], totalMonthCancel[this.todayMonth]);
      this.createCardValueSales(totalValue);
      this.createCardValueSalesDiff(totalValuePerMonth[prevMonth.getMonth()], totalValuePerMonth[this.todayMonth]);
      Graph.createGraphLine(params);
    } catch (error) {
      if (error instanceof Error) {
        alertService('error', error.message, 'red');
      }
    }
  }

  public createGraphSalesMonthOrigin(dashboard: any) {
    try {
      const PreviousMonthParams = {
        months: this.retroMonths,
        includeCurrentMonth: true,
      };
      const data = DateTime.getPreviousMonths(PreviousMonthParams);
      const legendData: any = [];
      const today = new Date();
      const todayYear = today.getFullYear();
      const datasource: any = [];
      const nextMonth = new Date();
      nextMonth.setDate(1);
      nextMonth.setMonth(nextMonth.getMonth() - this.retroMonths);

      const salesOriginPerMonth = dashboard.data.data.reduce((acc: any, cv: any) => {
        const [day, month, year] = this.getDayMonthYear(cv.DTHRABERMESA);
        const intMonth = parseInt(month, 10) - 1;
        acc[intMonth] = acc[intMonth] || {};
        if (day && month && parseInt(year, 10) === todayYear && cv.IDSTCOMANDA !== '4') {
          if (!legendData.includes(cv.IDORGCMDVENDA)) {
            const newSeries = {
              name: `${cv.IDORGCMDVENDA}`,
              type: 'line',
              data: this.createArray(12),
              label: {
                show: true,
                position: 'top',
                color: 'black',
                fontSize: 12,
                formatter(d: { data: string; }) {
                  return `${d.data}`;
                },
              },
            };
            datasource.push(newSeries);
            legendData.push(cv.IDORGCMDVENDA);
          }
          acc[intMonth][cv.IDORGCMDVENDA] = acc[intMonth][cv.IDORGCMDVENDA] + 1 || 1;
        }
        return acc;
      }, {});

      let k = 0;
      for (nextMonth; nextMonth.getMonth() !== this.todayMonth + 1; nextMonth.setMonth(nextMonth.getMonth() + 1), k += 1) {
        let j = 0;
        let totalMonth = 0;
        for (j; j < legendData.length; j += 1) {
          if (salesOriginPerMonth[nextMonth.getMonth()]) {
            totalMonth += salesOriginPerMonth[nextMonth.getMonth()][legendData[j]] || 0;
            datasource[j].data[k] = salesOriginPerMonth[nextMonth.getMonth()][legendData[j]] || 0;
          } else {
            datasource[j].data[k] = 0;
          }
        }
        data[k] = data[k].concat(`(${totalMonth})`);
      }

      const title = 'Pedidos Mensais por Origem';
      const name = 'sales origin per month';
      const id = 'dashboardCard17';
      const tooltip = '';

      const params = {
        datasource,
        title,
        name,
        id,
        tooltip,
        legendData,
        data,
      };
      Graph.createGraphLine(params);
    } catch (error) {
      if (error instanceof Error) {
        alertService('error', error.message, 'red');
      }
    }
  }

  public createGraphSalesMonthOriginCancel(dashboard: any) {
    try {
      const PreviousMonthParams = {
        months: this.retroMonths,
        includeCurrentMonth: true,
      };
      const data = DateTime.getPreviousMonths(PreviousMonthParams);
      const legendData: any = [];
      const today = new Date();
      const todayYear = today.getFullYear();
      const datasource: any = [];
      let totalValueCancel = 0;
      let totalCancel = 0;
      const totalValuePerMonth: number[] = this.createArray(12);
      const nextMonth = new Date();
      nextMonth.setDate(1);
      nextMonth.setMonth(nextMonth.getMonth() - this.retroMonths);
      const salesOriginPerMonth = dashboard.data.data.reduce((acc: any, cv: any) => {
        const [day, month, year] = this.getDayMonthYear(cv.DTHRABERMESA);
        const intMonth = parseInt(month, 10) - 1;
        acc[intMonth] = acc[intMonth] || {};
        if (day && month && parseInt(year, 10) === todayYear && cv.IDSTCOMANDA === '4') {
          if (!legendData.includes(cv.IDORGCMDVENDA)) {
            const newSeries = {
              name: `${cv.IDORGCMDVENDA}`,
              type: 'line',
              data: this.createArray(12),
              label: {
                show: true,
                position: 'top',
                color: 'black',
                fontSize: 12,
                formatter(d: { data: string; }) {
                  return `${d.data}`;
                },
              },
            };
            datasource.push(newSeries);
            legendData.push(cv.IDORGCMDVENDA);
          }
          totalCancel += 1;
          totalValueCancel += parseFloat(cv.VRPEDIDO);
          totalValuePerMonth[parseInt(month, 10) - 1] += parseFloat(cv.VRPEDIDO);
          acc[intMonth][cv.IDORGCMDVENDA] = acc[intMonth][cv.IDORGCMDVENDA] + 1 || 1;
        }
        return acc;
      }, {});

      let k = 0;
      const totalMonthCancel: number[] = this.createArray(12);
      for (nextMonth; nextMonth.getMonth() !== this.todayMonth + 1; nextMonth.setMonth(nextMonth.getMonth() + 1), k += 1) {
        let j = 0;
        for (j; j < legendData.length; j += 1) {
          if (salesOriginPerMonth[nextMonth.getMonth()]) {
            datasource[j].data[k] = salesOriginPerMonth[nextMonth.getMonth()][legendData[j]] || 0;
            totalMonthCancel[nextMonth.getMonth()] += salesOriginPerMonth[nextMonth.getMonth()][legendData[j]] || 0;
          } else {
            datasource[j].data[k] = 0;
          }
        }
        data[k] = data[k].concat(`(${totalMonthCancel[nextMonth.getMonth()]})`);
      }

      const prevMonth = new Date();
      prevMonth.setMonth(prevMonth.getMonth() - 1);

      const title = 'Cancelamentos Mensais por Origem';
      const name = 'cancel origin per month';
      const id = 'dashboardCard19';
      const tooltip = '';

      const params = {
        datasource,
        title,
        name,
        id,
        tooltip,
        legendData,
        data,
      };

      this.createCardQuantCancel(totalCancel);
      this.createCardQuantCancelDiff(totalMonthCancel[prevMonth.getMonth()], totalMonthCancel[this.todayMonth]);
      this.createCardValueCancel(totalValueCancel);
      this.createCardValueCancelDiff(totalValuePerMonth[prevMonth.getMonth()], totalValuePerMonth[this.todayMonth]);
      Graph.createGraphLine(params);
    } catch (error) {
      if (error instanceof Error) {
        alertService('error', error.message, 'red');
      }
    }
  }

  public createGraphSalesDelay(dashboard: any) {
    try {
      const PreviousMonthParams = {
        months: this.retroMonths,
        includeCurrentMonth: true,
      };
      const data = DateTime.getPreviousMonths(PreviousMonthParams);
      const datasource = [{
        name: 'Pedidos Atrasados',
        type: 'line',
        data: this.createArray(12),
        label: {
          show: true,
          position: 'top',
          color: 'black',
          fontSize: 12,
          formatter(d: { data: string; }) {
            return `${d.data}`;
          },
        },
      }];

      const salesDelayed = dashboard.data.data.reduce((acc: any, cv: any) => {
        const [day, month, year] = this.getDayMonthYear(cv.DTHRABERMESA);
        const timeHour = cv.DTHRABERMESA.split(' ');
        const endSale = cv.DTHRFECHMESA || new Date();
        const timeExpected = new Date(`${month}/${day}/${year} ${timeHour[1]}`);
        timeExpected.setMinutes(timeExpected.getMinutes() + parseInt(cv.TEMPOPEDIDO, 10));
        const intMonth = parseInt(month, 10) - 1;
        acc[intMonth] = acc[intMonth] || {};
        if (endSale > timeExpected && cv.IDSTCOMANDA < 4) {
          acc[intMonth] = parseInt(acc[intMonth], 10) + 1 || 1;
        }
        return acc;
      }, {});
      const nextMonth = new Date();
      nextMonth.setMonth(nextMonth.getMonth() - this.retroMonths);
      let k = 0;
      for (nextMonth; nextMonth.getMonth() !== this.todayMonth + 1; nextMonth.setMonth(nextMonth.getMonth() + 1), k += 1) {
        datasource[0].data[k] = salesDelayed[nextMonth.getMonth()] || 0;
        data[k] = data[k].concat(`(${datasource[0].data[k]})`);
      }

      const title = 'Atrasos Mensais';
      const name = 'delay monthly';
      const id = 'dashboardCard20';
      const tooltip = '';
      const legendData = ['Pedidos Atrasados'];

      const params = {
        datasource,
        title,
        name,
        id,
        tooltip,
        legendData,
        data,
      };

      Graph.createGraphLine(params);
    } catch (error) {
      if (error instanceof Error) {
        alertService('error', error.message, 'red');
      }
    }
  }

  public createGraphSalesHour(dashboard: any) {
    try {
      const data = ['0:00', '1:00', '2:00', '3:00', '4:00', '5:00', '6:00', '7:00', '8:00', '9:00', '10:00', '11:00',
        '12:00', '13:00', '14:00', '15:00', '16:00', '17:00', '18:00', '19:00', '20:00', '21:00', '22:00', '23:00'];
      const datasource = [{
        name: 'Pedidos',
        type: 'line',
        data: this.createArray(24),
        label: {
          show: true,
          position: 'top',
          color: 'black',
          fontSize: 12,
          formatter(d: { data: string; }) {
            return `${d.data}`;
          },
        },
      }];

      dashboard.data.data.forEach((element: any) => {
        const timeHour = element.DTHRABERMESA.split(' ');
        const partHour = timeHour[1].split(':');
        const hour = parseInt(partHour[0], 10);
        if (element.IDSTCOMANDA !== '4') {
          datasource[0].data[hour] += 1;
        }
      });

      const title = 'Horário dos Pedidos';
      const name = 'sales per hour';
      const id = 'dashboardCard16';
      const color = ['#e9561c'];
      const tooltip = '';
      const legendData = ['Pedidos'];

      const params = {
        datasource,
        title,
        name,
        id,
        color,
        tooltip,
        legendData,
        data,
      };
      Graph.createGraphLine(params);
    } catch (error) {
      if (error instanceof Error) {
        alertService('error', error.message, 'red');
      }
    }
  }

  public createGraphSalesDelayHour(dashboard: any) {
    try {
      const data = ['0:00', '1:00', '2:00', '3:00', '4:00', '5:00', '6:00', '7:00', '8:00', '9:00', '10:00', '11:00',
        '12:00', '13:00', '14:00', '15:00', '16:00', '17:00', '18:00', '19:00', '20:00', '21:00', '22:00', '23:00'];
      const datasource = [{
        name: 'Pedidos Atrasados',
        type: 'line',
        data: this.createArray(24),
        label: {
          show: true,
          position: 'top',
          color: 'black',
          fontSize: 12,
          formatter(d: { data: string; }) {
            return `${d.data}`;
          },
        },
      }];

      dashboard.data.data.forEach((element: any) => {
        const [day, month, year] = this.getDayMonthYear(element.DTHRABERMESA);
        const timeHour = element.DTHRABERMESA.split(' ');
        const partHour = timeHour[1].split(':');
        const hour = parseInt(partHour[0], 10);
        const minute = parseInt(partHour[1], 10);
        const second = parseInt(partHour[2], 10);
        const endSale = element.DTHRFECHMESA || new Date();
        const timeExpected = new Date(`${month}/${day}/${year} ${hour}:${minute}:${second}`);
        timeExpected.setMinutes(timeExpected.getMinutes() + element.TEMPOPEDIDO);
        if (endSale > timeExpected && element.IDSTCOMANDA < 4) {
          datasource[0].data[hour] += 1;
        }
      });

      const title = 'Horário dos Pedidos Atrasados';
      const name = 'cancel per hour';
      const id = 'dashboardCard21';
      const tooltip = '';
      const legendData = ['Pedidos Atrasados'];

      const params = {
        datasource,
        title,
        name,
        id,
        tooltip,
        legendData,
        data,
      };

      Graph.createGraphLine(params);
    } catch (error) {
      if (error instanceof Error) {
        alertService('error', error.message, 'red');
      }
    }
  }

  public createCardQuantSales(quant: number) {
    const formattedValue = quant.toString().replace(/\B(?=(\d{3})+(?!\d))/g, '.');
    this.quantSales = `${formattedValue}`;
  }

  public createCardValueSales(quant: number) {
    const formattedValue = quant.toFixed(2).replace('.', ',');
    const [integerPart, decimalPart] = formattedValue.split(',');
    const integerPartWithThousand = integerPart.replace(/\B(?=(\d{3})+(?!\d))/g, '.');
    this.valueSales = `R$ ${integerPartWithThousand},${decimalPart}`;
  }

  public createCardQuantCancel(quant: number) {
    const formattedValue = quant.toString().replace(/\B(?=(\d{3})+(?!\d))/g, '.');
    this.quantCancel = `${formattedValue}`;
  }

  public createCardQuantCancelDiff(lastMonth: number, currMonth: number) {
    const formattedValue = (currMonth - lastMonth).toString().replace(/\B(?=(\d{3})+(?!\d))/g, '.');
    this.quantCancelDiff = `${formattedValue}`;
  }

  public createCardQuantSalesDiff(lastMonth: number, currMonth: number) {
    const formattedValue = (currMonth - lastMonth).toString().replace(/\B(?=(\d{3})+(?!\d))/g, '.');
    this.quantSalesDiff = `${formattedValue}`;
  }

  public createCardValueSalesDiff(lastMonth: number, currMonth: number) {
    const formattedValue = (currMonth - lastMonth).toFixed(2).replace('.', ',');
    const [integerPart, decimalPart] = formattedValue.split(',');
    const integerPartWithThousand = integerPart.replace(/\B(?=(\d{3})+(?!\d))/g, '.');
    this.valueSalesDiff = `R$ ${integerPartWithThousand},${decimalPart}`;
  }

  public createCardQuantSalesToday(quant: number) {
    const formattedValue = quant.toString().replace(/\B(?=(\d{3})+(?!\d))/g, '.');
    this.quantSalesToday = `<strong>${formattedValue}</strong>`;
  }

  public createCardValueSalesToday(quant: number) {
    const formattedValue = quant.toFixed(2).replace('.', ',');
    const [integerPart, decimalPart] = formattedValue.split(',');
    const integerPartWithThousand = integerPart.replace(/\B(?=(\d{3})+(?!\d))/g, '.');
    this.valueSalesToday = `R$ ${integerPartWithThousand},${decimalPart}`;
  }

  public createCardQuantCancelToday(quant: number) {
    const formattedValue = quant.toString().replace(/\B(?=(\d{3})+(?!\d))/g, '.');
    this.quantCancelToday = `${formattedValue}`;
  }

  public createCardValueCancel(quant: number) {
    const formattedValue = quant.toFixed(2).replace('.', ',');
    const [integerPart, decimalPart] = formattedValue.split(',');
    const integerPartWithThousand = integerPart.replace(/\B(?=(\d{3})+(?!\d))/g, '.');
    this.valueCancel = `R$ ${integerPartWithThousand},${decimalPart}`;
  }

  public createCardValueCancelDiff(lastMonth: number, currMonth: number) {
    const formattedValue = (currMonth - lastMonth).toFixed(2).replace('.', ',');
    const [integerPart, decimalPart] = formattedValue.split(',');
    const integerPartWithThousand = integerPart.replace(/\B(?=(\d{3})+(?!\d))/g, '.');
    this.valueCancelDiff = `R$ ${integerPartWithThousand},${decimalPart}`;
  }

  public createCardValueCancelToday(quant: number) {
    const formattedValue = quant.toFixed(2).replace('.', ',');
    const [integerPart, decimalPart] = formattedValue.split(',');
    const integerPartWithThousand = integerPart.replace(/\B(?=(\d{3})+(?!\d))/g, '.');
    this.valueCancelToday = `R$ ${integerPartWithThousand},${decimalPart}`;
  }

  private getDayMonthYear(DTHRABERMESA: string) {
    let [day, month, year] = DTHRABERMESA.split('/');
    day = day.replace(/^0+/, '');
    month = month.replace(/^0+/, '');
    year = year.replace(/^0+/, '');
    return [day, month, year.slice(0, 4)];
  }

  private createArray(size: number, defaultValue = 0) {
    return [...Array(size)].fill(defaultValue, 0, size);
  }

  private getMonthsAgo(monthsAgo: number): string {
    const today = new Date();
    const year = today.getFullYear();
    const month = today.getMonth() - monthsAgo;

    let adjustedYear = year;
    let adjustedMonth = month;

    if (adjustedMonth < 0) {
      adjustedMonth = 12 + adjustedMonth;
      adjustedYear -= 1;
    }

    const date = `1/${adjustedMonth + 1}/${adjustedYear} 00:00:00`;

    return date;
  }
}
