import Vue from 'vue';
import Router from 'vue-router';
import { components } from '@zeedhi/vuetify';

const { ZdFramePage } = components;

Vue.use(Router);

let isAuthenticated = false;

if (localStorage.getItem('TOKEN') !== null) {
  isAuthenticated = true;
}

const router = new Router({
  mode: 'history',
  base: process.env.BASE_URL,
  routes: [
    {
      path: '/',
      name: 'home',
      component: () => import('./views/Home.vue'),
    },
    {
      path: '/dashboard',
      name: 'dashboard',
      component: ZdFramePage,
      props: () => ({
        path: 'dashboard',
        local: true,
      }),
    },
    {
      name: 'main-page',
      path: '/main-page',
      component: ZdFramePage,
      props: {
        path: 'main-page',
        local: true,
      },
    },
    {
      path: '*',
      component: ZdFramePage,
      props: {
        path: 'notfound',
        local: true,
      },
    },
    {
      name: 'logs',
      path: '/logs',
      component: ZdFramePage,
      props: {
        path: 'logs',
        local: true,
      },
    },
    {
      name: 'login',
      path: '/login',
      component: ZdFramePage,
      props: {
        path: 'login',
        local: true,
      },
    },
    {
      name: 'reset-password-page',
      path: '/reset-password-page',
      component: ZdFramePage,
      props: {
        path: 'reset-password-page',
        local: true,
      },
    },
  ],
});

router.beforeEach((to, from, next) => {
  if (to.name === 'reset-password-page') {
    next();
  } else if (to.name !== 'login' && isAuthenticated === false) {
    next({
      name: 'login',
      query: {
        redirect: to.path,
      },
    });
  } else if (to.name === 'login' && isAuthenticated === true) {
    let redi:any = '/login';
    if (to.query.redirect !== undefined) {
      redi = to.query.redirect;
    }
    next({ path: redi });
  } else {
    next();
  }
});

const pageTitles: { [key: string]: string } = {
  '': 'Home',
  '/login': 'Login',
  '/dashboard': 'Dashboard',
  '/main-page': 'Acompanhamento de Pedidos',
  '/logs': 'Logs',
  '/reset-password-page': 'Alterar Senha',
};

router.afterEach((to) => {
  const route = to.path.replace(/\/$/, '');
  const pageTitle = pageTitles[route];

  if (pageTitle) {
    document.title = pageTitle;
  }
});

export default router;
