<?php
namespace App\Util;

class Util {
    private $connection;

    public function __construct (\Doctrine\DBAL\Connection $connection) {
        $this->connection = $connection;
    }
    
    public function newCode ($counterName, $size) {
        /* NOVOCODIGO */
        try {
            $nrseq = null;
            $connection = $this->connection;
            $storedProcedure = new StoredProcedure($connection, "NOVO_CODIGO_NT");
            $storedProcedure->addParam(new Param("P_CONTADOR", Param::PARAM_INPUT, $counterName, Param::PARAM_TYPE_STR, strlen($counterName)));
            $storedProcedure->addParam(new Param("P_SEQUENCIAL", Param::PARAM_OUTPUT, $nrseq, Param::PARAM_TYPE_INT, 20));
            $result = $storedProcedure->execute();
            return substr($result["P_SEQUENCIAL"], strlen($result["P_SEQUENCIAL"])-$size);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }
    
    /**
     *
     *
     * @param StoredProcedure $procedure The procedure to be executed.
     * @param array $paramValues Param values to override values present in Param instance.
     *
     * @return array The output parameters indexed by name.
     */
    public function executeProcedure(StoredProcedure $procedure, array $paramValues = array()) {
        $queryStmt = $this->getQueryString($procedure);
        $stmt = $this->connection->prepare($queryStmt);
        $valuesOutput = array();
        foreach ($procedure->getParams() as $param) {
            if ($param->isOutput()) {
                $valuesOutput[$param->getName()] = $param->getValue();
                $stmt->bindParam($param->getParamAlias(), $valuesOutput[$param->getName()], $param->getType(), $param->getLength());
            } else {
                $stmt->bindValue($param->getParamAlias(), $param->getValue(), $param->getType());
            }
        }

        $stmt->execute();
        return $valuesOutput;
    }

    /**
     * @param StoredProcedure $procedure
     * @return string
     */
    protected function getQueryString(StoredProcedure $procedure) {
        $paramNames = array_map(function (Param $param) {
            return ":" . $param->getName();
        }, $procedure->getParams());
        $paramString = implode(", ", $paramNames);
        return "CALL " . $procedure->getName() . " ( " . $paramString . " )";
    }

    public function inside($point, $vs) {
        $x = $point[0];
        $y = $point[1];

        $inside = false;
        for ($i = 0, $j = count($vs) - 1; $i < count($vs); $j = $i++) {
            $xi = (float)$vs[$i][0];
            $yi = (float)$vs[$i][1];
            $xj = (float)$vs[$j][0];
            $yj = (float)$vs[$j][1];

            $intersect = (($yi > $y) != ($yj > $y)) && ($x < ($xj - $xi) * ($y - $yi) / ($yj - $yi) + $xi);
            if ($intersect)
                $inside = !$inside;
        }
        return $inside;
    }

    public function removeAcentos($texto) {
        return preg_replace(array("/(á|à|ã|â|ä)/","/(Á|À|Ã|Â|Ä)/","/(é|è|ê|ë)/","/(É|È|Ê|Ë)/","/(í|ì|î|ï)/","/(Í|Ì|Î|Ï)/","/(ó|ò|õ|ô|ö)/","/(Ó|Ò|Õ|Ô|Ö)/","/(ú|ù|û|ü)/","/(Ú|Ù|Û|Ü)/","/(ñ)/","/(Ñ)/","/(ç)/","/(Ç)/"),explode(" ","a A e E i I o O u U n N c C"),$texto);
    }

    public function getEnv($name) {
        return $_ENV[$name];
    }

    public function getConfigs() {
        return json_decode(file_get_contents(__DIR__ . '/../../config/configs.json'));
    }

    public static function encrypt($password) {
        return md5($password);
    }

    public static function getDelphiPasswordEncrypted($password, $isEncrypt = true) {
        $url = $_ENV['ASP_URL'] . '/comwebservice/service.asmx?wsdl';

        $options = array(
            'encoding'    => 'ISO-8859-1',
            'trace'       => true,
            'exceptions'  => true,
            'compression' => SOAP_COMPRESSION_ACCEPT | SOAP_COMPRESSION_GZIP          
        );

        $args = array(
            new \SoapVar($password, XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema")
        );
        
        $cryptType = $isEncrypt ? 'GetCriptoPass' : 'GetUnCriptoPass';
        $params    = array(
            'rdm'    => 'Bib_html.rdmDBGeralHtm',
            'method' => $cryptType,
            'args'   => $args,
        );

        try {
            $client     = new \SoapClient($url, $options);
            $result     = $client->executeCOMMethod($params);
            $encryption = $result->executeCOMMethodResult;
            return $encryption;
        } catch (\Exception $e) {     
            throw new \Exception($e->getMessage());
        }
    }
}

